#!/usr/bin/python3

import argparse
import sys
import config
import primary
import secondary

if __name__ == '__main__':
    argParser = argparse.ArgumentParser(usage="%(prog)s [options]")
    group = argParser.add_mutually_exclusive_group(required=True)
    group.add_argument('--primary', nargs='*',
                       metavar='CONFIG',
                       help='backup as specified in CONFIG')
    group.add_argument('--secondary', nargs=3,
                       metavar=('SOCKET', 'MOUNT', 'PATH'),
                       help='backup from SOCKET to MOUNT/PATH2')
    argParser.add_argument('--xattr',  action='store_true',
                           help='let md5toc store MD5 in extended attribute')
    argParser.add_argument('--max-age', 
                           metavar='SECONDS',
                           help='maximum SECONDS since last MD5 calculation')
    argParser.add_argument('--user', 
                           metavar='USER',
                           help='ssh USER for secondary (-l)')
    argParser.add_argument('--identity', 
                           metavar='IDENTITY',
                           help='ssh IDENTITY for secondary (-i)')
    argParser.add_argument('--debug', action='store_true',
                           help='debug actions')
 
    options = argParser.parse_args(sys.argv[1:])

    if options.primary:
        done = set()
        for path in options.primary:
            if path in done:
                continue
            done.add(path)
            primary.do_backup(options=options,
                              config=config.parse(open(path).read()))
            
    if options.secondary:
        secondary.do_backup(options=options,
                            socket_path=options.secondary[0],
                            mount=options.secondary[1],
                            path=options.secondary[2])

